/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.view

import android.app.Activity
import android.content.pm.ActivityInfo
import android.os.Bundle
import android.os.Parcelable
import android.os.ResultReceiver
import androidx.appcompat.app.AppCompatActivity
import androidx.lifecycle.ViewModelProvider
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.friend.Constants
import com.kakao.sdk.partner.friend.Repository
import com.kakao.sdk.partner.friend.model.RequestPickerParams
import com.kakao.sdk.partner.friend.model.ViewAppearance
import com.kakao.sdk.partner.friend.util.LayoutUtils
import com.kakao.sdk.partner.friend.viewmodel.FriendViewModel
import com.kakao.sdk.partner.friend.viewmodel.FriendViewModelFactory
import com.kakao.sdk.v2.partner.friend.R

open class PickerActivity : AppCompatActivity() {
    private lateinit var resultReceiver: ResultReceiver

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        try {
            intent.extras?.getBundle(Constants.KEY_BUNDLE)?.run {
                resultReceiver =
                    getParcelable<ResultReceiver>(Constants.KEY_RESULT_RECEIVER) as ResultReceiver
                val param = getParcelable<Parcelable>(Constants.KEY_PARAM)
                val isSingle = getBoolean(Constants.KEY_IS_SINGLE)
                val isPopup = getBoolean(Constants.KEY_IS_POPUP)
                param?.let {
                    ViewModelProvider(
                        viewModelStore,
                        FriendViewModelFactory(application, resultReceiver, it, isSingle, isPopup)
                    ).get(FriendViewModel::class.java)
                }
                (param as RequestPickerParams).let {
                    if (it.viewAppearance == ViewAppearance.LIGHT) {
                        setTheme(if (isPopup) R.style.DayOnlyPopup else R.style.DayOnlyFull)
                    } else if (it.viewAppearance == ViewAppearance.DARK) {
                        setTheme(if (isPopup) R.style.NightOnlyPopup else R.style.NightOnlyFull)
                    }

                    if (it.enableLandScape == false) {
                        requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT
                    }
                }
                LayoutUtils.initialize(applicationContext, isPopup)
            }
        } catch (e: Throwable) {
            SdkLog.e(e)
            sendError(ClientError(ClientErrorCause.Unknown).apply { initCause(e) })
        }

        if (savedInstanceState == null) {
            supportFragmentManager.beginTransaction()
                .add(R.id.fragment_container, FriendPickerFragment()).commit()
        }
    }

    override fun onBackPressed() {
        super.onBackPressed()
        sendError(ClientError(ClientErrorCause.Cancelled, "cancelled."))
        Repository.clearData()
    }

    private fun sendError(exception: KakaoSdkError) {
        if (this::resultReceiver.isInitialized) {
            resultReceiver.send(
                Activity.RESULT_CANCELED,
                Bundle().apply {
                    putSerializable(Constants.KEY_EXCEPTION, exception)
                }
            )
        }
        finish()
    }
}